@@ -33,8 +33,8 @@ enum {
 	BLOCKING,
 	LAST_SIGNAL
 };
-static guint   signals [LAST_SIGNAL];
-static GList  *cnx_list = NULL;
+static guint  signals [LAST_SIGNAL];
+static GList *cnx_list = NULL;
 
 #define CNX_LOCK(cnx)    G_STMT_START { link_lock();   } G_STMT_END
 #define CNX_UNLOCK(cnx)  G_STMT_START { link_unlock(); } G_STMT_END
@@ -109,12 +109,69 @@ link_close_fd (LinkConnection *cnx)
 }
 
 typedef struct {
-	guchar       *data;
+	LinkBrokenCallback fn;
+	gpointer           user_data;
+} BrokenCallback;
 
-	struct iovec *vecs;
-	int           nvecs;
-	struct iovec  single_vec;
-} QueuedWrite;
+static void
+link_connection_emit_broken (LinkConnection *cnx, GSList *callbacks)
+{
+	GSList *l;
+
+	for (l = callbacks; l; l = l->next) {
+		BrokenCallback *bc = l->data;
+		bc->fn (cnx, bc->user_data);
+		g_free (bc);
+	}
+	g_slist_free (callbacks);
+}
+
+static gboolean
+link_connection_broken_idle (gpointer data)
+{
+	GSList *callbacks;
+	LinkConnection *cnx = data;
+
+	d_printf ("Connection %p broken idle ...\n", data);
+
+	CNX_LOCK (cnx);
+	callbacks = cnx->idle_broken_callbacks;
+	cnx->idle_broken_callbacks = NULL;
+	cnx->inhibit_reconnect = FALSE;
+	link_signal ();
+	CNX_UNLOCK (cnx);
+
+	link_connection_emit_broken (cnx, callbacks);
+
+	link_connection_unref (cnx);
+
+	return FALSE;
+}
+
+static void
+link_source_remove (LinkConnection *cnx)
+{
+	if (cnx->priv->tag) {
+		LinkWatch *thewatch = cnx->priv->tag;
+		cnx->priv->tag = NULL;
+		link_io_remove_watch (thewatch);
+		d_printf ("Removed watch on %d\n", cnx->priv->fd);
+	}
+}
+
+static void
+link_source_add (LinkConnection *cnx,
+		 GIOCondition    condition)
+{
+	g_assert (cnx->priv->tag == NULL);
+
+	cnx->priv->tag = link_io_add_watch_fd (
+		cnx->priv->fd, condition,
+		link_connection_io_handler, cnx);
+
+	d_printf ("Added watch on %d (0x%x)\n",
+		 cnx->priv->fd, condition);
+}
 
 /*
  * link_connection_class_state_changed:
@@ -175,11 +232,35 @@ link_connection_state_changed_T_R (LinkConnection      *cnx,
 	case LINK_DISCONNECTED:
 		link_source_remove (cnx);
 		link_close_fd (cnx);
+		queue_free (cnx);
 		/* don't free pending queue - we could get re-connected */
 		if (changed) {
+
+			d_printf ("Emitting the broken signal on %p\n", cnx);
 			CNX_UNLOCK (cnx);
 			g_signal_emit (cnx, signals [BROKEN], 0);
 			CNX_LOCK (cnx);
+
+			if (cnx->idle_broken_callbacks) {
+				if (!link_thread_io ()) {
+					GSList *callbacks;
+
+					d_printf ("Immediate broken callbacks at immediately\n");
+
+					callbacks = cnx->idle_broken_callbacks;
+					cnx->idle_broken_callbacks = NULL;
+
+					CNX_UNLOCK (cnx);
+					link_connection_emit_broken (cnx, callbacks);
+					CNX_LOCK (cnx);
+				} else {
+					d_printf ("Queuing broken callbacks at idle\n");
+
+					cnx->inhibit_reconnect = TRUE;
+					link_connection_ref_T (cnx);
+					g_idle_add (link_connection_broken_idle, cnx);
+				}
+			}
 		}
 		break;
 	}
@@ -187,6 +268,7 @@ link_connection_state_changed_T_R (LinkConnection      *cnx,
 	klass = (LinkConnectionClass *)G_OBJECT_GET_CLASS (cnx);
 
 	if (klass->state_changed) {
+		link_signal ();
 		CNX_UNLOCK (cnx);
 		klass->state_changed (cnx, status);
 		CNX_LOCK (cnx);
@@ -237,6 +319,14 @@ calc_size (struct iovec *src_vecs,
 	return total_size;
 }
 
+typedef struct {
+	guchar       *data;
+
+	struct iovec *vecs;
+	int           nvecs;
+	struct iovec  single_vec;
+} QueuedWrite;
+
 static void
 queue_flattened_T_R (LinkConnection *cnx,
 		     struct iovec   *src_vecs,
@@ -303,31 +393,6 @@ queue_free (LinkConnection *cnx)
 	cnx->priv->write_queue = NULL;
 }
 
-static void
-link_source_remove (LinkConnection *cnx)
-{
-	if (cnx->priv->tag) {
-		LinkWatch *thewatch = cnx->priv->tag;
-		cnx->priv->tag = NULL;
-		link_io_remove_watch (thewatch);
-		d_printf ("Removed watch on %d\n", cnx->priv->fd);
-	}
-}
-
-static void
-link_source_add (LinkConnection *cnx,
-		 GIOCondition    condition)
-{
-	g_assert (cnx->priv->tag == NULL);
-
-	cnx->priv->tag = link_io_add_watch_fd (
-		cnx->priv->fd, condition,
-		link_connection_io_handler, cnx);
-
-	d_printf ("Added watch on %d (0x%x)\n",
-		 cnx->priv->fd, condition);
-}
-
 static void
 link_connection_from_fd_T (LinkConnection         *cnx,
 			   int                     fd,
@@ -344,7 +409,9 @@ link_connection_from_fd_T (LinkConnection         *cnx,
 	cnx->options       = options;
 	cnx->priv->fd      = fd;
 
+	g_free (cnx->remote_host_info);
 	cnx->remote_host_info = remote_host_info;
+	g_free (cnx->remote_serv_info);
 	cnx->remote_serv_info = remote_serv_info;
 
 	d_printf ("Cnx from fd (%d) '%s', '%s', '%s'\n",
@@ -472,6 +539,44 @@ link_connection_do_initiate (LinkConnection        *cnx,
 	return retval;
 }
 
+static LinkConnectionStatus
+link_connection_wait_connected_T (LinkConnection *cnx)
+{
+	while (cnx && cnx->status == LINK_CONNECTING)
+		link_wait ();
+
+	return cnx ? cnx->status : LINK_DISCONNECTED;
+}
+
+LinkConnectionStatus
+link_connection_try_reconnect (LinkConnection *cnx)
+{
+	LinkConnectionStatus status;
+
+	g_return_val_if_fail (LINK_IS_CONNECTION (cnx), LINK_DISCONNECTED);
+
+	CNX_LOCK (cnx);
+
+	d_printf ("Try for reconnection on %p: %d\n",
+		  cnx, cnx->inhibit_reconnect);
+
+	while (cnx->inhibit_reconnect)
+		link_wait ();
+
+	if (cnx->status != LINK_DISCONNECTED)
+		g_warning ("trying to re-connect connected cnx.");
+	else
+		link_connection_do_initiate
+			(cnx, cnx->proto->name, cnx->remote_host_info,
+			 cnx->remote_serv_info, cnx->options);
+
+	status = link_connection_wait_connected_T (cnx);
+
+	CNX_UNLOCK (cnx);
+
+	return status;
+}
+
 /**
  * link_connection_initiate_list:
  * @derived_type: a #LinkConnection derived type
@@ -993,10 +1098,15 @@ link_connection_dispose (GObject *obj)
 static void
 link_connection_finalize (GObject *obj)
 {
+	GSList *l;
 	LinkConnection *cnx = (LinkConnection *)obj;
 
 	link_close_fd (cnx);
 
+	for (l = cnx->idle_broken_callbacks; l; l = l->next)
+		g_free (l->data);
+	g_slist_free (cnx->idle_broken_callbacks);
+
 	g_free (cnx->remote_host_info);
 	g_free (cnx->remote_serv_info);
 
@@ -1108,7 +1218,7 @@ link_connection_io_handler (GIOChannel  *gioc,
 	LinkConnection      *cnx = data;
 	LinkConnectionClass *klass;
 
-	d_printf ("linc_connection_io_handler fd %d, 0x%x\n",
+	d_printf ("link_connection_io_handler fd %d, 0x%x\n",
 		  cnx->priv->fd, condition);
 
 	CNX_LOCK (cnx);
@@ -1216,13 +1326,15 @@ link_connection_disconnect (LinkConnection *cnx)
 LinkConnectionStatus
 link_connection_wait_connected (LinkConnection *cnx)
 {
-	g_return_val_if_fail (!link_thread_safe (),
-			      LINK_CONNECTED);
-	
-	while (cnx && cnx->status == LINK_CONNECTING) 
-		link_main_iteration (TRUE);
+	LinkConnectionStatus status;
 
-	return cnx ? cnx->status : LINK_DISCONNECTED;
+	CNX_LOCK (cnx);
+
+	status = link_connection_wait_connected_T (cnx);
+
+	CNX_UNLOCK (cnx);
+
+	return status;
 }
 
 void
@@ -1234,3 +1346,49 @@ link_connections_move_io_T (gboolean to_io_thread)
 		link_watch_move_io (cnx->priv->tag, to_io_thread);
 	}
 }
+
+void
+link_connection_add_broken_cb (LinkConnection    *cnx,
+			       LinkBrokenCallback fn,
+			       gpointer           user_data)
+{
+	BrokenCallback *bc = g_new0 (BrokenCallback, 1);
+
+	g_return_if_fail (fn != NULL);
+
+	bc->fn = fn;
+	bc->user_data = user_data;
+
+	cnx->idle_broken_callbacks = g_slist_prepend (cnx->idle_broken_callbacks, bc);
+}
+
+static gboolean
+broken_callback_match (BrokenCallback    *bc,
+		       LinkBrokenCallback fn,
+		       gpointer           user_data)
+{
+	return ( (!fn || bc->fn == fn) &&
+		 (!user_data || bc->user_data == user_data) );
+}
+
+void
+link_connection_remove_broken_cb (LinkConnection    *cnx,
+				  LinkBrokenCallback opt_fn,
+				  gpointer           opt_user_data)
+{
+	GSList *l, *next;
+
+	CNX_LOCK (cnx);
+
+	for (l = cnx->idle_broken_callbacks; l; l = next) {
+		next = l->next;
+		if (broken_callback_match (l->data, opt_fn, opt_user_data)) {
+			g_free (l->data);
+			cnx->idle_broken_callbacks =
+				g_slist_delete_link (cnx->idle_broken_callbacks,
+						     l);
+		}
+	}	
+
+	CNX_UNLOCK (cnx);
+}
