@@ -16,7 +16,7 @@
 
 static void IOP_profile_free (IOP_Profile_info *p);
 
-#ifdef LINC_SSL_SUPPORT
+#ifdef LINK_SSL_SUPPORT
 static IOP_Component_info *
 IOP_component_find (GSList *list, IOP_ComponentId type, GSList **pos)
 {
@@ -29,7 +29,7 @@ IOP_component_find (GSList *list, IOP_ComponentId type, GSList **pos)
 
   return NULL;
 }
-#endif /* LINC_SSL_SUPPORT */
+#endif /* LINK_SSL_SUPPORT */
 
 static gchar *
 IOP_ObjectKey_dump (ORBit_ObjectKey *objkey)
@@ -229,7 +229,7 @@ IOP_profile_get_info (CORBA_Object  obj,
       *host = iiop->host;
       *service = tmpbuf;
       g_snprintf(tmpbuf, 8, "%d", iiop->port);
-#ifdef LINC_SSL_SUPPORT
+#ifdef LINK_SSL_SUPPORT
       {
 	IOP_TAG_SSL_SEC_TRANS_info *ssli;
 	ssli = (IOP_TAG_SSL_SEC_TRANS_info *)
@@ -249,7 +249,7 @@ IOP_profile_get_info (CORBA_Object  obj,
       *proto = giop->proto;
       *host = giop->host;
       *service = giop->service;
-#ifdef LINC_SSL_SUPPORT
+#ifdef LINK_SSL_SUPPORT
       {
 	IOP_TAG_GENERIC_SSL_SEC_TRANS_info *ssli;
 	ssli = (IOP_TAG_GENERIC_SSL_SEC_TRANS_info *)
@@ -458,11 +458,13 @@ IOP_delete_profiles (CORBA_ORB orb,
 		return;
 
 	if (orb && *profiles == orb->profiles)
-		return;
-
-	g_slist_foreach (*profiles, (GFunc)IOP_profile_free, NULL);
-	g_slist_free (*profiles);
-	*profiles = NULL;
+		*profiles = NULL;
+	else {
+		g_slist_foreach (*profiles,
+				 (GFunc)IOP_profile_free, NULL);
+		g_slist_free (*profiles);
+		*profiles = NULL;
+	}
 }
 
 GSList *
@@ -476,16 +478,16 @@ IOP_start_profiles (CORBA_ORB orb)
 	IOP_TAG_INTERNET_IOP_info        *iiop = NULL;
 
 	for (l = orb->servers ; l != NULL ; l = l->next) {
-		LINCServer *serv = l->data;
+		LinkServer *serv = l->data;
 		gboolean   ipv4, ipv6, uds, ssl;
 
 		ipv4 = !strcmp (serv->proto->name, "IPv4");
 		ipv6 = !strcmp (serv->proto->name, "IPv6");
 		uds  = !strcmp (serv->proto->name, "UNIX");
 
-		ssl  = (serv->create_options & LINC_CONNECTION_SSL);
+		ssl  = (serv->create_options & LINK_CONNECTION_SSL);
 
-		if (!osi && (uds || (ipv6 && !ssl))) {
+		if (!osi && uds) {
 			osi = g_new0 (IOP_TAG_ORBIT_SPECIFIC_info, 1);
 			osi->parent.profile_type = IOP_TAG_ORBIT_SPECIFIC;
 		}
@@ -493,9 +495,6 @@ IOP_start_profiles (CORBA_ORB orb)
 		if (uds && !osi->unix_sock_path)
 			osi->unix_sock_path = g_strdup (serv->local_serv_info);
 
-		if (ipv6 && !ssl)
-			osi->ipv6_port = atoi (serv->local_serv_info);
-
 		if (ipv4) {
 			if (!iiop) {
 				iiop = g_new0 (IOP_TAG_INTERNET_IOP_info, 1);
@@ -527,7 +526,7 @@ IOP_start_profiles (CORBA_ORB orb)
 
 				giopt = l2->data;
 				if (giopt->parent.profile_type == IOP_TAG_GENERIC_IOP &&
-				    strcmp (giopt->proto, serv->proto->name)) {
+				   !strcmp (giopt->proto, serv->proto->name)) {
 					giop = giopt;
 					break;
 				}
@@ -613,8 +612,6 @@ IOP_generate_profiles (CORBA_Object obj)
 		obj->object_key = ORBit_OAObject_object_to_objkey (adaptor_obj);
 
 	obj->profile_list = orb->profiles;
-
-	ORBit_register_objref (obj);
 }
 
 static void
