@@ -66,14 +66,17 @@ testConst (void)
 static void
 testSequenceHelpers (void)
 {
-	CORBA_long l;
-	test_BoundedLongSeq *lseq;
+	CORBA_long           l    = 0;
+	test_BoundedLongSeq *lseq = NULL;
 
 	lseq = ORBit_sequence_alloc (TC_test_BoundedLongSeq, 2);
 	g_assert (lseq != NULL);
-	g_assert (lseq->_length == 2);
+	g_assert (lseq->_length ==  2);
 	g_assert (lseq->_maximum >= 2);
 
+	ORBit_sequence_index (lseq, 0) = 0;
+	ORBit_sequence_index (lseq, 1) = 0;
+	
 	l = 1;
 	ORBit_sequence_append (lseq, &l);
 	l++;
@@ -87,6 +90,43 @@ testSequenceHelpers (void)
 	ORBit_sequence_set_size (lseq, 0);
 
 	CORBA_free (lseq);
+
+	/* test incremental reallocation of memory */
+	{
+		CORBA_long i = 0;
+		CORBA_long j = 0;
+
+		/* 16 times double capacity of buffer */
+		CORBA_long MAX_APPEND = 1<<16; 
+		/* generic sequence<octet> test */ 
+		CORBA_sequence_CORBA_octet *oseq = NULL;
+		
+		oseq = ORBit_sequence_alloc (TC_CORBA_sequence_CORBA_octet, 0);
+		
+		g_assert (oseq != NULL);
+		g_assert (oseq->_length == 0);
+		g_assert (oseq->_maximum >= 0);
+
+		for (i = 0; i < MAX_APPEND; ++i)
+		{
+			CORBA_octet oct = (CORBA_octet) (i % 109);
+			/* _append does shallow copy */ 
+			ORBit_sequence_append (oseq, &oct); /* realloc */ 
+			g_assert (i+1==oseq->_length);
+			
+			/* infrequent validation sequence values have
+			 * been re-located correctly */
+			if (i % 367 == 0) 
+			for (j = 0; j < oseq->_length; ++j /* prim */ )
+			{
+				CORBA_octet j_check = (CORBA_octet) (j % 109);
+				CORBA_octet j_value = ORBit_sequence_index (oseq, j);
+				g_assert (j_value==j_check);
+			}
+		}
+		
+		CORBA_free (oseq);
+	}
 }
 
 static void
@@ -540,6 +580,7 @@ testUnboundedSequence (test_TestFactory   factory,
 {
   test_SequenceServer objref;
   test_StrSeq *outArg = NULL, inArg, inoutArg, *retn;
+  /*  test_LongSeq *long_retn; */
   guint i;
   d_print ("Testing unbounded sequences...\n");
   objref = test_TestFactory_getSequenceServer (factory, ev);
@@ -576,6 +617,11 @@ testUnboundedSequence (test_TestFactory   factory,
   for (i=0;i<retn->_length;i++)
 	g_assert (strcmp (retn->_buffer[i], constants_SEQ_STRING_RETN[i]) == 0);
 
+  g_warning ("FIXME: opMassiveSeq fails - due to max. size check");
+  /*  long_retn = test_SequenceServer_opMassiveSeq(objref, ev);
+      g_assert (ev->_major == CORBA_NO_EXCEPTION);
+      CORBA_free (long_retn); */
+
   CORBA_free (inArg._buffer);
   CORBA_free (inoutArg._buffer);
   CORBA_free (outArg);
@@ -1597,7 +1643,9 @@ testPingPong (test_TestFactory   factory,
 
 #if NUM_THREADS > 0
 	if (thread_tests) {
-		g_warning ("No thread available to handle incoming requests");
+		static volatile int warned = 0;
+		if (!warned++)
+			g_warning ("No thread available to handle incoming requests");
 		return;
 	}
 #endif
@@ -1709,6 +1757,8 @@ testPingPong (test_TestFactory   factory,
 /*		g_assert (broken); */
 		CORBA_exception_free (ev);
 		ORBit_small_connection_unref (cnx);
+
+		ORBit_connection_set_max_buffer (cnx, 0);
 	}
 
 	CORBA_Object_release (objref, ev);
@@ -1731,41 +1781,44 @@ static void
 testSegv (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
+	gboolean broken = FALSE;
+	gboolean invoked = FALSE;
+
+	if (in_proc) 
+		return;
+
 	d_print ("Testing Fatal invocations ...\n");
-	if (!in_proc) {
-		gboolean broken = FALSE;
-		gboolean invoked = FALSE;
 
-		g_assert (ORBit_small_listen_for_broken (
-			factory, G_CALLBACK (broken_cb), &broken) ==
-			  ORBIT_CONNECTION_CONNECTED);
+	g_assert (ORBit_small_listen_for_broken (
+						 factory, G_CALLBACK (broken_cb), &broken) ==
+		  ORBIT_CONNECTION_CONNECTED);
 
-		g_assert (ORBit_small_listen_for_broken (
-			factory, G_CALLBACK (dummy_cb), &invoked) ==
-			  ORBIT_CONNECTION_CONNECTED);
+	g_assert (ORBit_small_listen_for_broken (
+						 factory, G_CALLBACK (dummy_cb), &invoked) ==
+		  ORBIT_CONNECTION_CONNECTED);
 
-		g_assert (ORBit_small_unlisten_for_broken (
-			factory, G_CALLBACK (dummy_cb)) ==
-			  ORBIT_CONNECTION_CONNECTED);
+	g_assert (ORBit_small_unlisten_for_broken (
+						   factory, G_CALLBACK (dummy_cb)) ==
+		  ORBIT_CONNECTION_CONNECTED);
+	g_assert (!CORBA_Object_non_existent (factory, ev));
 
-		test_TestFactory_segv (factory, "do it!", ev); 
-#ifdef DO_HARDER_SEGV
-		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
-		g_assert (!strcmp (ev->_id, "IDL:omg.org/CORBA/COMM_FAILURE:1.0"));
-		CORBA_exception_free (ev);
+	test_TestFactory_segv (factory, "do it!", ev); 
 
-		g_assert (ORBit_small_get_connection_status (factory) ==
-			  ORBIT_CONNECTION_DISCONNECTED);
-		g_assert (broken);
-		g_assert (!invoked);
+#ifdef DO_HARDER_SEGV // unusual
+	g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+	g_assert (!strcmp (ev->_id, "IDL:omg.org/CORBA/COMM_FAILURE:1.0"));
+	CORBA_exception_free (ev);
+
+	g_assert (ORBit_small_get_connection_status (factory) ==
+		  ORBIT_CONNECTION_DISCONNECTED);
+	g_assert (broken);
+	g_assert (!invoked);
 #else
-		if (ORBit_small_unlisten_for_broken (
-			factory, G_CALLBACK (broken_cb)) !=
-		    ORBIT_CONNECTION_CONNECTED)
-			g_warning ("Unusual race in unlisten");
-		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	if (ORBit_small_unlisten_for_broken (factory, G_CALLBACK (broken_cb)) !=
+	    ORBIT_CONNECTION_CONNECTED)
+		g_warning ("Unusual race in unlisten");
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 #endif
-	}
 }
 
 static void
@@ -1824,7 +1877,7 @@ test_time_noop (test_TestFactory   factory,
 	g_timer_elapsed (timer, NULL) * 1000);
 
 	old_flags = ORBit_small_flags;
-	ORBit_small_flags != ORBIT_SMALL_FORCE_GENERIC_MARSHAL;
+	ORBit_small_flags |= ORBIT_SMALL_FORCE_GENERIC_MARSHAL;
 	g_timer_reset (timer);
 	g_timer_start (timer);
 	for (i = 0; i < TIME_TEST_RUNS; i++)
@@ -1900,6 +1953,34 @@ testDerivedServer (test_TestFactory   factory,
 	CORBA_free (oid);	
 }
 
+static void
+testNonExistent (test_TestFactory factory, CORBA_Environment *ev)
+{
+	CORBA_Object non_existent;
+	const char *non_existent_ior =
+		"IOR:010000001f00000049444c3a6f726269742f746573742f54657"
+		"374466163746f72793a312e300000030000000054424f6400000001"
+		"01020005000000554e495800000000160000006c6f63616c686f737"
+		"42e6c6f63616c646f6d61696e0000002d0000002f746d702f6f7262"
+		"69742d6d69636861656c2f6c696e632d363733322d302d373362323"
+		"966373333316662390000000000000000caaedfba58000000010102"
+		"002d0000002f746d702f6f726269742d6d69636861656c2f6c696e6"
+		"32d363733322d302d37336232396637333331666239000000001c00"
+		"000000000000331c40f8ba0fa828dc2928282828282808000000db7"
+		"e269601000000480000000100000002000000050000001c00000000"
+		"000000331c40f8ba0fa828dc2928282828282808000000db7e26960"
+		"1000000140000000100000001000105000000000901010000000000";
+
+	if (!in_proc)
+		return;
+	d_print ("Testing CORBA_Object_non_existent ...\n");
+
+	non_existent = CORBA_ORB_string_to_object
+		(global_orb, non_existent_ior, ev);
+	g_assert (CORBA_Object_non_existent (non_existent, ev));
+	CORBA_Object_release (non_existent, NULL);
+}
+
 static void
 run_tests (test_TestFactory   factory, 
 	   gboolean           thread_tests,
@@ -1935,6 +2016,7 @@ run_tests (test_TestFactory   factory,
 		testContext (factory, ev);
 		testIInterface (factory, ev);
 		testDerivedServer (factory, ev);
+		testNonExistent (factory, ev);
 #ifndef TIMING_RUN
 		if (!thread_tests)
 			testAsync (factory, ev);
